# Copyright (c) vhffs project and its contributors
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
#2. Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in
#   the documentation and/or other materials provided with the
#   distribution.
#3. Neither the name of vhffs nor the names of its contributors
#   may be used to endorse or promote products derived from this
#   software without specific prior written permission.
#
#THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
#COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
#INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
#BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
#CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

use strict;
use warnings;
use utf8;

use Vhffs::Constants;
use Vhffs::Functions;
use Vhffs::Robots;
use Vhffs::Tls;

package Vhffs::Robots::Tls;

sub _create {
	my ($tls, $err, $output) = @_;
	return undef unless defined $tls and defined $err;

	my $ret;
	my $vhffs = $tls->get_vhffs;
	my $tlsconf = $vhffs->get_config->get_tls;

	my $dir = $tls->get_dir;
	unless (-d $dir) {
		File::Path::make_path( $dir, { error => \my $errors });
		if (@$errors) {
			$$err = join(': ', %{@$errors[0]});
			return undef;
		}
	}

	my $privkeypath = $dir.'/privkey.pem';

	require IPC::Open3;
	my $outfd;
	if (defined $output) {
		require Symbol;
		$outfd = Symbol::gensym;
		$$output = '';
	} else {
		require File::Spec;
		open ($outfd, '>', File::Spec->devnull);
	}

	# Unfortunately there is no way to silence openssl for genpkey, whatever you do, it will still
	# display its '.' and '+' progress characters while generating a key, thus we have to discard stderr.
	my $oldumask = umask 0077;
	my $childpid = IPC::Open3::open3(undef, $outfd, $outfd, split(' ', $tlsconf->{'openssl_genpkey_cmd'}), '-out', $privkeypath);
	if ($childpid) {
		# Capture child output
		if (defined $output) {
			while (<$outfd>) { $$output .= $_ }
		}

		# wait for the child to finish
		waitpid ($childpid, 0);

		# $? contains the return value, The high byte is the exit value of the process. The low 7 bits represent
		# the number of the signal that killed the process, with the 8th bit indicating whether a core dump occurred.
		# -- signal is 0 if no signal were sent to kill the process
		# -- exit value is 0 if the process succeed
		# -- core dump bit is 0 if no core dump were written to disk
		# ---- so, $? contains 0 if everything went fine
		$ret = $? >> 8 if $?;
	}
	umask $oldumask;

	# Something went wrong
	unless ($childpid and not defined $ret and -s $privkeypath) {
		$$err = 'private key generation failed';
		return undef;
	}

	return 1;
}

sub _gencsr {
	my ($tls, $domains, $csrtmppath, $err, $output) = @_;
	return undef unless defined $tls and defined $domains and defined $csrtmppath and defined $err;

	my $ret;
	my $vhffs = $tls->get_vhffs;
	my $tlsconf = $vhffs->get_config->get_tls;

	my $dir = $tls->get_dir;
	my $privkeypath = $dir.'/privkey.pem';

	# Create the OpenSSL temporary config file for generation of certificate request
	#
	# Unfortunately we cannot pass additional domains using the OpenSSL command line, therefore we
	# have to create a temporary OpenSSL configuration file.
	#
	my ($cfgfileout, $cfgtmppath) = Vhffs::Robots::tmpfile($vhffs);
	unless (defined $cfgfileout) {
		$$err = 'temporary OpenSSL configuration failed';
		return undef;
	}

	open(my $cfgfilein, '<', $tlsconf->{'openssl_req_cfg'});

	while (<$cfgfilein>) {
		print $cfgfileout $_;
	}

	print $cfgfileout '[VHFFS-subjectAltName]'."\n".'subjectAltName=';
	my @d;
	foreach (@$domains) {
		push @d, 'DNS:'.$_;
	}
	print $cfgfileout join(',', @d)."\n";

	close($cfgfileout);
	close($cfgfilein);

	# Generate certificate request
	require IPC::Open3;
	my $outfd;
	if (defined $output) {
		require Symbol;
		$outfd = Symbol::gensym;
		$$output = '';
	} else {
		require File::Spec;
		open ($outfd, '>', File::Spec->devnull);
	}

	my $childpid = IPC::Open3::open3(undef, $outfd, $outfd, split(' ', $tlsconf->{'openssl_req_cmd'}),
		'-reqexts', 'VHFFS-subjectAltName', '-config', $cfgtmppath, '-key', $privkeypath, '-out', $csrtmppath);
	if ($childpid) {
		# Capture child output
		if (defined $output) {
			while (<$outfd>) { $$output .= $_ }
		}

		# wait for the child to finish
		waitpid ($childpid, 0);

		# $? contains the return value, The high byte is the exit value of the process. The low 7 bits represent
		# the number of the signal that killed the process, with the 8th bit indicating whether a core dump occurred.
		# -- signal is 0 if no signal were sent to kill the process
		# -- exit value is 0 if the process succeed
		# -- core dump bit is 0 if no core dump were written to disk
		# ---- so, $? contains 0 if everything went fine
		$ret = $? >> 8 if $?;
	}
	unlink $cfgtmppath;

	# Something went wrong
	unless ($childpid and not defined $ret) {
		$$err = 'certificate request generation failed';
		return undef;
	}

	return 1;
}

sub _certbot_dryrun {
	my ($tls, $csrtmppath, $err, $output) = @_;
	return undef unless defined $tls and defined $csrtmppath and defined $err;

	my $ret;
	my $vhffs = $tls->get_vhffs;
	my $tlsconf = $vhffs->get_config->get_tls;

	# Try with the Let's Encrypt staging API, that's not really nice for Let's Encrypt API servers
	# but their rate limiting is very harsh, and we couldn't find a way to prevent behing blacklisted,
	# especially the limit about the max failed requests/week (300 failed requests/week at the time of writing).
	#
	# The staging API is not rate limited but deliver certificates that are not working, but it tells
	# us whether the validation will succeed using the real API :-)
	require IPC::Open3;
	my $outfd;
	if (defined $output) {
		require Symbol;
		$outfd = Symbol::gensym;
		$$output = '';
	} else {
		require File::Spec;
		open ($outfd, '>', File::Spec->devnull);
	}

	my $childpid = IPC::Open3::open3(undef, $outfd, $outfd, split(' ', $tlsconf->{'certbot_cert_cmd'}), '--non-interactive',
		'--staging', '--dry-run',
		'--csr', $csrtmppath );
	if ($childpid) {
		# Capture child output
		if (defined $output) {
			while (<$outfd>) { $$output .= $_ }
		}

		# wait for the child to finish
		waitpid ($childpid, 0);

		# $? contains the return value, The high byte is the exit value of the process. The low 7 bits represent
		# the number of the signal that killed the process, with the 8th bit indicating whether a core dump occurred.
		# -- signal is 0 if no signal were sent to kill the process
		# -- exit value is 0 if the process succeed
		# -- core dump bit is 0 if no core dump were written to disk
		# ---- so, $? contains 0 if everything went fine
		$ret = $? >> 8 if $?;
	}

	# Something went wrong
	unless ($childpid and not defined $ret) {
		$$err = 'Let\'s Encrypt certificate request failed (dry-run)';
		return undef;
	}

	return 1;
}

sub create_or_renew {
	my $tls = shift;

	my $ret;
	my $err;
	my $output;
	my $created = 0;
	my $errmsg;
	my $vhffs = $tls->get_vhffs;
	my $tlsconf = $vhffs->get_config->get_tls;

	unless (-f $tlsconf->{'openssl_req_cfg'}) {
		print 'ERROR: cannot find OpenSSL default configuration file for certificate generation at '.$tlsconf->{'openssl_req_cfg'}."\n";
		return undef;
	}

	my $dir = $tls->get_dir;
	my $privkeypath = $dir.'/privkey.pem';

	unless (-d $dir and -f $privkeypath) {
		unless (_create($tls, \$err, \$output)) {
			$errmsg = 'An error occured while creating TLS certificate: '.$err;
			goto failed;
		}
		$created = 1;
	}

	$errmsg = 'An error occured while '.($created ? 'creating' : 'renewing').' TLS certificate: '.join(', ', sort keys %{$tls->get_domains}).': ';

	if ($tlsconf->{'use_certbot'}) {

		# FIXME, we should pass the $csrtmpfile to _gencsr instead of $csrtmppath

		# Generate certificate request
		my ($csrtmpfile, $csrtmppath) = Vhffs::Robots::tmpfile($vhffs);
		unless (defined $csrtmpfile) {
			$errmsg .= 'temporary CSR creation failed';
			goto failed;
		}
		close ($csrtmpfile);

		# First, try to generate the certificate with all domains
		my $domainlist = { %{$tls->get_domains} };  # Shallow copy
		# Filter domain list based on exclude configuration
		my @regexp = split ' ', $tlsconf->{'exclude'};
		foreach my $d (keys %{$domainlist}) {
			foreach my $r (@regexp) {
				if ($d =~ /$r/) {
					delete $domainlist->{$d};
					last;
				}
			}
		}
		unless (_gencsr($tls, [sort keys %{$domainlist}], $csrtmppath, \$err, \$output)) {
			$errmsg .= $err;
			unlink $csrtmppath;
			goto failed;
		}

		# Try to generate the certificate using the Let's Encrypt staging API
		unless (_certbot_dryrun($tls, $csrtmppath, \$err, \$output)) {
			# store full error message
			$errmsg .= $err;

			# Try to generate a partial certificate by checking each domain individually
			foreach my $d (values %{$domainlist}) {
				# We don't care about the output here, that's meaningless for the user

				unless (_gencsr($tls, [$d->get_domain], $csrtmppath, \$err, undef)) {
					$errmsg .= $err;
					unlink $csrtmppath;
					goto failed;
				}

				unless (_certbot_dryrun($tls, $csrtmppath, \$err, undef)) {
					delete $domainlist->{$d->get_domain};
				}
			}

			# None of the domain validated
			unless (scalar keys %{$domainlist}) {
				unlink $csrtmppath;
				goto failed;
			}

			# Generate the CSR with partial domains list
			unless (_gencsr($tls, [sort keys %{$domainlist}], $csrtmppath, \$err, \$output)) {
				$errmsg .= $err;
				unlink $csrtmppath;
				goto failed;
			}

			# Try to generate the partial certificate using the Let's Encrypt staging API
			unless (_certbot_dryrun($tls, $csrtmppath, \$err, \$output)) {
				unlink $csrtmppath;
				goto failed;
			}
		}

		# Generating the certificate using the staging API worked, try with the real API
		require IPC::Open3;
		require Symbol;
		my $outfd = Symbol::gensym;
		$output = '';

		# Generate certificate using a random name then move it to ensure the atomicness of operation
		# FIXME, now that certbot generate predictable paths, we can probably use Vhffs::Robots::tmpfile instead
		my $easyfindrand = join('', (0..9, 'A'..'Z', 'a'..'z')[map {rand 62} (1..8)]);
		my $lecertpath = $dir.'/'.$easyfindrand.'_cert.pem';
		my $lechainpath = $dir.'/'.$easyfindrand.'_chain.pem';
		my $lefullchainpath = $dir.'/'.$easyfindrand.'_fullchain.pem';

		my $childpid = IPC::Open3::open3(undef, $outfd, $outfd, split(' ', $tlsconf->{'certbot_cert_cmd'}), '--non-interactive',
			'--cert-path', $lecertpath,
			'--chain-path', $lechainpath,
			'--fullchain-path', $lefullchainpath,
			'--csr', $csrtmppath );
		if ($childpid) {
			# Capture child output
			while (<$outfd>) { $output .= $_ }

			# wait for the child to finish
			waitpid ($childpid, 0);

			# $? contains the return value, The high byte is the exit value of the process. The low 7 bits represent
			# the number of the signal that killed the process, with the 8th bit indicating whether a core dump occurred.
			# -- signal is 0 if no signal were sent to kill the process
			# -- exit value is 0 if the process succeed
			# -- core dump bit is 0 if no core dump were written to disk
			# ---- so, $? contains 0 if everything went fine
			$ret = $? >> 8 if $?;
		}
		unlink $csrtmppath;

		# Something went wrong
		unless ($childpid and not defined $ret and -s $lecertpath and -s $lechainpath and -s $lefullchainpath) {
			$errmsg .= 'Let\'s Encrypt certificate request failed';
			unlink $lecertpath;
			unlink $lechainpath;
			unlink $lefullchainpath;
			goto failed;
		}

		# Finally, move certificate to its final place
		require File::Copy;
		File::Copy::move($lecertpath, $dir.'/cert.pem');
		File::Copy::move($lechainpath, $dir.'/chain.pem');
		File::Copy::move($lefullchainpath, $dir.'/fullchain.pem');

		# Update expiration date
		my $expirestr;
		my ($opensslbin) = split(' ', $tlsconf->{'openssl_req_cmd'});
		$childpid = open( my $output, '-|', $opensslbin, 'x509', '-enddate', '-noout', '-in', $dir.'/cert.pem');
		if ($childpid) {
			# Read "notAfter" value from openssl output
			while (<$output>) { $expirestr = $_; };

			# wait for the child to finish
			waitpid ($childpid, 0);

			# $? contains the return value, The high byte is the exit value of the process. The low 7 bits represent
			# the number of the signal that killed the process, with the 8th bit indicating whether a core dump occurred.
			# -- signal is 0 if no signal were sent to kill the process
			# -- exit value is 0 if the process succeed
			# -- core dump bit is 0 if no core dump were written to disk
			# ---- so, $? contains 0 if everything went fine
			$ret = $? >> 8 if $?;
		}

		# Parse expiration date
		require Date::Parse;
		my ($date) = ($expirestr =~ /^notAfter=(.*)/);
		my $timestamp = Date::Parse::str2time($date);

		# Something went wrong
		unless ($childpid and not defined $ret and $timestamp > 0) {
			$errmsg .= 'Failed to fetch certificate expiration date';
			goto failed;
		}

		$tls->set_expire($timestamp);
		# Yeah, this check is only here for the sake of completeness :-)
		my $beforedays =  $tlsconf->{'invalid_before'} // 2;
		$tls->set_valid($timestamp > time() + 86400*$beforedays);

		# Set whether we included the domain into the certificate (in case of partial certificate)
		foreach (values %{$tls->get_domains}) {
			$_->set_valid(exists($domainlist->{$_->get_domain})?1:0);
		}
	} else {
		# Yes, it means we actually do nothing it if certbot is not enabled
		$tls->set_valid(0);
	}

	$tls->set_retries(0);
	$tls->set_output(undef);
	$tls->commit;

	my @dtext;
	foreach (sort keys %{$tls->get_domains}) {
		$_ = $tls->get_domains->{$_};
		push @dtext, $_->get_domain.($_->get_valid ? '' : '(F)');
	}
	Vhffs::Robots::vhffs_log($vhffs, ($created ? 'Created' : 'Renewed').' TLS certificate: '.join(', ', @dtext));

	nginx_proxy_conf($vhffs);
	return 1;

failed:
	Vhffs::Robots::vhffs_log($vhffs, $errmsg.', retries: '.$tls->get_retries);
	$tls->set_retries( $tls->get_retries +1 );
	$tls->set_output( $output );
	$tls->commit;
	return undef;
}

sub invalidate {
	my $tls = shift;

	my $vhffs = $tls->get_vhffs;

	# Certificate will expire soon, invalidate

	$tls->set_valid(0);
	$tls->commit;
	Vhffs::Robots::vhffs_log($vhffs, 'Invalidated TLS certificate: '.join(', ', sort keys %{$tls->get_domains}));

	nginx_proxy_conf($vhffs);
	return 1;
}

sub delete {
	my $tls = shift;

	my $vhffs = $tls->get_vhffs;
	my $dir = $tls->get_dir;

	require File::Path;
	require File::Basename;
	File::Path::remove_tree( $dir, { error => \my $errors });
	my $parent = File::Basename::dirname($dir);
	rmdir $parent;
	$parent = File::Basename::dirname($parent);
	rmdir $parent;

	if (@$errors) {
		Vhffs::Robots::vhffs_log($vhffs, 'An error occured while removing TLS certificate: '.join(', ', sort keys %{$tls->get_domains}).': '.join(': ', %{@$errors[0]}));
		return undef;
	}

	if ($tls->destroy) {
		Vhffs::Robots::vhffs_log($vhffs, 'Deleted TLS certificate '.join(', ', sort keys %{$tls->get_domains}));
	} else {
		Vhffs::Robots::vhffs_log($vhffs, 'An error occured while removing TLS certificate: '.join(', ', sort keys %{$tls->get_domains}));
		return undef;
	}

	nginx_proxy_conf($vhffs);
	return 1;
}

sub nginx_proxy_conf {
	require File::Copy;

	my $vhffs = shift;

	my $confdir = $vhffs->get_config->get_datadir.'/tls/conf';
	unless (-d $confdir) {
		mkdir( $confdir ) or Vhffs::Robots::vhffs_log( $vhffs, 'Unable to create svn confdir '.$confdir.': '.$! );
	}

	my ($tmpfile, $tmppath) = Vhffs::Robots::tmpfile( $vhffs );
	return undef unless defined $tmpfile;

	my $tlss = Vhffs::Tls::get_valid_list($vhffs);
	foreach my $tls (@$tlss) {
		my $dir = $tls->get_dir;
		print $tmpfile 'server {'."\n";
		foreach (sort keys %{$tls->get_domains}) {
			my $d = $tls->get_domains->{$_};
			next unless $d->get_valid;
			print $tmpfile '    server_name '.$d->get_domain.';'."\n";
		}
		print $tmpfile "\n";
		print $tmpfile '    ssl_certificate '.$dir.'/fullchain.pem;'."\n";
		print $tmpfile '    ssl_certificate_key '.$dir.'/privkey.pem;'."\n";
		print $tmpfile "\n";
		print $tmpfile '    include snippets/ssl-proxy.conf;'."\n";
		print $tmpfile '}'."\n\n";
	}

	close( $tmpfile );

	chmod 0644, $tmppath;
	File::Copy::move( $tmppath, $confdir.'/nginx-tls-proxy.conf' );
	return 1;
}

1;
