# Copyright (c) vhffs project and its contributors
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
#2. Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in
#   the documentation and/or other materials provided with the
#   distribution.
#3. Neither the name of vhffs nor the names of its contributors
#   may be used to endorse or promote products derived from this
#   software without specific prior written permission.
#
#THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
#COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
#INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
#BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
#CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

use strict;
use warnings;
use utf8;

use Vhffs::Functions;

=head1 NAME

Vhffs::Tls - Class allowing TLS certificates manipulation

=head1 SYNOPSIS



=head1 CLASS METHODS

=cut
package Vhffs::Tls::Domain;

=head2 _new

	Self constructor, almost private, please use get_by_* methods instead.

=cut
sub _new {
	my ($class, $domain_id, $domain, $valid) = @_;

	my $self = {};

	bless($self, $class);

	$self->{domain_id} = $domain_id;
	$self->{domain} = $domain;
	$self->{valid} = $valid;

	return $self;
}

=cut

=pod

=head2 get_domain

Get the domain domain name.

=cut
sub get_domain {
	my $self = shift;
	return $self->{'domain'};
}

=cut

=pod

=head2 set_valid

Set the domain validity state.

=cut
sub set_valid {
	my ( $self , $value ) = @_;
	$self->{'valid'} = $value;
}

=cut

=pod

=head2 get_valid

Get the domain validity state.

=cut
sub get_valid {
	my $self = shift;
	return $self->{'valid'};
}


package Vhffs::Tls;

=pod

=head2 _new

	Self constructor, almost private, please use get_by_* methods instead.

=cut
sub _new {
	my ($class, $vhffs, $tls_id, $retries, $expire, $persistent, $domainshash, $valid, $output, $domains) = @_;

	my $self = {};

	bless($self, $class);

	$self->{vhffs} = $vhffs;
	$self->{tls_id} = $tls_id;
	$self->{retries} = $retries;
	$self->{expire} = $expire;
	$self->{persistent} = $persistent;
	$self->{domainshash} = $domainshash;
	$self->{valid} = $valid;
	$self->{output} = $output;
	$self->{domains} = $domains // {};

	return $self;
}

=head2 create

my $tag = Vhffs::Tls::create($vhffs, $domains)

Create (in database) and return a new object.

=over 4

=item C<$vhffs>: C<Vhffs> instance

=item C<$domains>: An arrayref to the certificate domain list.

=back

=cut
sub create {
	my ($vhffs, $domains) = @_;

	return undef unless ref $domains eq 'ARRAY' and @$domains;
	foreach (@$domains) {
		return undef unless Vhffs::Functions::check_domain_name($_);
	}

	# Re-use previously generated certificate if available
	my $tls = get_by_domain_list($vhffs, $domains);
	if (defined $tls) {
		# be gentle, reset retries count so we can renew this re-used certificate as soon as possible if necessary
		$tls->set_retries(0);
		$tls->commit;
		return $tls;
	}

	my $dbh = $vhffs->get_db;
	local $dbh->{RaiseError} = 1;
	local $dbh->{PrintError} = 0;

	# Handle nested transactions
	my $transaction_started = 0;
	if ($dbh->{AutoCommit}) {
		# AutoCommit is on => we are not yet in a transaction, let's start one
		$dbh->begin_work;
		$transaction_started = 1;
        }

	eval {
		my $sql = q{INSERT INTO vhffs_tls(domainshash) VALUES(?) RETURNING tls_id, retries, expire, persistent, domainshash, valid, output};
		my $sth = $dbh->prepare($sql);
		$sth->execute(_generate_domainshash(@$domains));

		$tls = _new Vhffs::Tls($vhffs, $sth->fetchrow_array);

		foreach (@$domains) {
			$sql = q{INSERT INTO vhffs_tls_domain(tls_id, domain) VALUES(?, ?) RETURNING domain_id, domain, valid};
			$sth = $dbh->prepare($sql);
			$sth->execute($tls->{tls_id}, $_);

			$tls->{domains}->{$_} = _new Vhffs::Tls::Domain($sth->fetchrow_array);
		}

		$dbh->commit if $transaction_started;
	};

	if ($@) {
		warn 'Unable to create TLS certificate for '.join(', ', @$domains).': '.$@."\n";
		$dbh->rollback if $transaction_started;
		return undef;
	}

	return $tls;
}

=cut

=head2 Vhffs::Tls:_fetch_domains

Private function, please use get_by_* methods instead.

=cut
sub _fetch_domains {
	my $tls = shift;

	my $dbh = $tls->get_vhffs->get_db;
	my $sql = q{SELECT domain_id, domain, valid FROM vhffs_tls_domain WHERE tls_id = ?};
	my $sth = $dbh->prepare($sql);
	$sth->execute($tls->{tls_id}) or return undef;

	while (my @d = $sth->fetchrow_array) {
		my $domain = _new Vhffs::Tls::Domain(@d);
		$tls->{domains}->{$domain->get_domain} = $domain;
	}
	return $tls->{domains};
}

=cut

=head2 Vhffs::Tls:get_by_tls_id

my $tls = Vhffs::Tls::get_by_tls_id( $vhffs, $tls_id );

Fetches a TLS certificate by its TLS ID.

=cut
sub get_by_tls_id {
	my ($vhffs, $tls_id) = @_;

	my $dbh = $vhffs->get_db;
	my $sql = q{SELECT tls_id, retries, expire, persistent, domainshash, valid, output FROM vhffs_tls WHERE tls_id = ?};
	my $sth = $dbh->prepare($sql);
	$sth->execute($tls_id) or return undef;
	my @results = $sth->fetchrow_array();
	return undef unless @results;

	my $tls = _new Vhffs::Tls($vhffs, @results);
	$tls->_fetch_domains;
	return $tls;
}

=cut

=head2 Vhffs::Tls:get_by_oid

my $tls = Vhffs::Tls::get_by_oid( $vhffs, $oid );

Fetches a TLS certificate by its related object ID.

=cut
sub get_by_oid {
	my ($vhffs, $oid) = @_;

	my $dbh = $vhffs->get_db;
	my $sql = q{SELECT t.tls_id, t.retries, t.expire, t.persistent, t.domainshash, t.valid, t.output FROM vhffs_tls t INNER JOIN vhffs_object o ON t.tls_id=o.tls_id WHERE o.object_id = ?};
	my $sth = $dbh->prepare($sql);
	$sth->execute($oid) or return undef;
	my @results = $sth->fetchrow_array();
	return undef unless @results;

	my $tls = _new Vhffs::Tls($vhffs, @results);
	$tls->_fetch_domains;
	return $tls;
}

=cut

=head2 Vhffs::Tls:get_by_domain_list

my $tls = Vhffs::Tls::get_by_domainlist( $vhffs, $domainlist );

Fetches a TLS certificate by its domain list.

=cut
sub get_by_domain_list {
	my ($vhffs, $domainlist) = @_;

	my $dbh = $vhffs->get_db;
	my $sql = q{SELECT tls_id, retries, expire, persistent, domainshash, valid, output FROM vhffs_tls WHERE domainshash = ?};
	my $sth = $dbh->prepare($sql);
	$sth->execute(_generate_domainshash(@$domainlist)) or return undef;
	my @results = $sth->fetchrow_array();
	return undef unless @results;

	my $tls = _new Vhffs::Tls($vhffs, @results);
	my $domains = $tls->_fetch_domains;

	# Check if hash collision occured, this is VERY VERY VERY unlikely to happen, this check is only here for the sake of completeness
	my %copy = %$domains;
	foreach (@$domainlist) {
		return undef unless exists $copy{$_};
		delete $copy{$_};
	}
	return undef if scalar keys %copy;

	return $tls;
}

=cut

=head2 Vhffs::Tls:get_renewal_list

my $tls = Vhffs::Tls::get_renewal_list($vhffs)

Returns an arrayref of all TLS certificates that need to be renewed.

=cut
sub get_renewal_list {
	my $vhffs = shift;

	my $tls = [];
	my $tlsconf = $vhffs->get_config->get_tls;
	my $beforedays = $tlsconf->{'renew_before'} // 30;
	my $maxretries =  $tlsconf->{'max_retries'} // 90;

	my @params;
	my $sql = 'SELECT tls_id, retries, expire, persistent, domainshash, valid, output FROM vhffs_tls
		WHERE (persistent OR tls_id IN (SELECT tls_id FROM vhffs_object WHERE state=?)) AND (expire < ? OR expire IS NULL) AND retries < ?';
	push @params, (time() + $beforedays*86400), $maxretries;

	# prefilter domains, we still have to let domains passes, for example if foo.tld is filtered but
	# bar.tld is not, a domain list of (www.foo.tld, www.bar.tld) must be accepted, therefore we
	# only filter domain lists where all domains are excluded
	if (defined $tlsconf->{'exclude'}) {
		$sql .= ' AND tls_id IN (SELECT tls_id FROM vhffs_tls_domain WHERE 1=1';
		foreach (split ' ', $tlsconf->{'exclude'}) {
			$sql .= ' AND domain !~* ?';
			push @params, $_;
		}
		$sql .= ')';
	}

	my $dbh = $vhffs->get_db;
	my $sth = $dbh->prepare($sql);
	$sth->execute(Vhffs::Constants::ACTIVATED, @params) or return undef;

	while (my $s = $sth->fetchrow_arrayref()) {
		my $t = _new Vhffs::Tls($vhffs, @$s);
		$t->_fetch_domains;
		push(@$tls, $t);
	}
	return $tls;
}

=cut

=head2 Vhffs::Tls:get_orphaned_and_expired_list

my $tls = Vhffs::Tls::get_orphaned_and_expired_list($vhffs);

Returns an arrayref of all TLS certificates orphaned and expired.

=cut
sub get_orphaned_and_expired_list {
	my $vhffs = shift;

	my $tls = [];

	my $sql = 'SELECT t.tls_id, t.retries, t.expire, t.persistent, t.domainshash, t.valid, t.output FROM vhffs_tls t
		LEFT JOIN vhffs_object o ON o.tls_id=t.tls_id WHERE o.tls_id IS NULL
		AND NOT t.persistent AND (t.expire < ? OR t.expire IS NULL)';

	my $dbh = $vhffs->get_db;
	my $sth = $dbh->prepare($sql);
	$sth->execute(time()) or return undef;

	while (my $s = $sth->fetchrow_arrayref()) {
		my $t = _new Vhffs::Tls($vhffs, @$s);
		$t->_fetch_domains;
		push(@$tls, $t);
	}
	return $tls;
}

=cut

=head2 Vhffs::Tls:get_valid_to_invalid_list

my $tls = Vhffs::Tls::get_valid_to_invalid_list($vhffs);

Returns an arrayref of all TLS certificates that should now
be considered invalid.

=cut
sub get_valid_to_invalid_list {
	my $vhffs = shift;

	my $tls = [];
	my $tlsconf = $vhffs->get_config->get_tls;
	my $beforedays =  $tlsconf->{'invalid_before'} // 2;

	my $sql = 'SELECT tls_id, retries, expire, persistent, domainshash, valid, output FROM vhffs_tls
		WHERE valid=true AND (expire < ? OR expire IS NULL)';

	my $dbh = $vhffs->get_db;
	my $sth = $dbh->prepare($sql);
	$sth->execute(time() + $beforedays*86400) or return undef;

	while (my $s = $sth->fetchrow_arrayref()) {
		my $t = _new Vhffs::Tls($vhffs, @$s);
		$t->_fetch_domains;
		push(@$tls, $t);
	}
	return $tls;
}

=cut

=head2 Vhffs::Tls:get_valid_list

my $tls = Vhffs::Tls::get_valid_list($vhffs);

Returns an arrayref of all TLS certificates which can be used on production servers.

=cut
sub get_valid_list {
	my $vhffs = shift;

	my $tls = [];
	my $tlsconf = $vhffs->get_config->get_tls;
	my $beforedays = $tlsconf->{'invalid_before'} // 2;

	my $sql = 'SELECT tls_id, retries, expire, persistent, domainshash, valid, output FROM vhffs_tls
		WHERE valid=true AND expire > ?';

	my $dbh = $vhffs->get_db;
	my $sth = $dbh->prepare($sql);
	$sth->execute(time() + $beforedays*86400) or return undef;

	while (my $s = $sth->fetchrow_arrayref()) {
		my $t = _new Vhffs::Tls($vhffs, @$s);
		$t->_fetch_domains;
		push(@$tls, $t);
	}
	return $tls;
}

=cut

=head2 Vhffs::Tls:_generate_domainshash

Private function, return a domain hash used to ensure certificate uniqueness.

=cut
sub _generate_domainshash {
	require Digest::MD5;
	return Digest::MD5::md5_hex(join ' ', sort @_);
}

=cut

=pod

=head2 Vhffs::Tls::commit

Apply all changes that were made on this object. Returns undef value if failed, true if success.

=cut
sub commit {
	my $tls = shift;

	my $dbh = $tls->get_vhffs->get_db;
	local $dbh->{RaiseError} = 1;
	local $dbh->{PrintError} = 0;

	# Handle nested transactions
	my $transaction_started = 0;
	if ($dbh->{AutoCommit}) {
		# AutoCommit is on => we are not yet in a transaction, let's start one
		$dbh->begin_work;
		$transaction_started = 1;
        }

	eval {
		my $sql = 'UPDATE vhffs_tls SET retries=?, expire=?, persistent=?, valid=?, output=? WHERE tls_id=?';
		$dbh->do($sql, undef, $tls->{'retries'} , $tls->{'expire'}, $tls->{'persistent'}, $tls->{'valid'}, $tls->{'output'}, $tls->{'tls_id'});

		foreach (values %{$tls->{domains}}) {
			my $sql = 'UPDATE vhffs_tls_domain SET valid=? WHERE domain_id=?';
			$dbh->do($sql, undef, $_->{'valid'} , $_->{'domain_id'});
		}

		$dbh->commit if $transaction_started;
	};

	if ($@) {
		$dbh->rollback if $transaction_started;
		return undef;
	}

	return 1;
}

=cut

=pod

=head2 Vhffs::Tls::destroy

Remove TLS certificate from database.

=cut
sub destroy {
	my $tls = shift;

	my $sql = 'DELETE FROM vhffs_tls WHERE tls_id=?';
	return $tls->get_vhffs->get_db->do($sql, undef, $tls->{'tls_id'});
}

=cut

=pod

=head2 get_vhffs

Return our main Vhffs object.

=cut
sub get_vhffs {
	my $self = shift;
	return $self->{'vhffs'};
}

=cut

=pod

=head2 get_tls_id

Get TLS id.

=cut
sub get_tls_id {
	my $self = shift;
	return $self->{'tls_id'};
}

=cut

=pod

=head2 get_domains

Return an hashref of Vhffs::Tls::Domain of all TLS certificate domains.

=cut
sub get_domains {
	my $self = shift;
	return $self->{'domains'};
}

=cut

=pod

=head2 get_dir

	my $dir = $tls->get_dir;

Returns TLS certificate directory.

=cut
sub get_dir {
	my $self = shift;
	my $hash = substr($self->{domainshash}, 0, 1).'/'.substr($self->{domainshash}, 1, 2).'/'.substr($self->{domainshash}, 3);
	return $self->get_vhffs->get_config->get_datadir.'/tls/'.$hash;
}

=cut

=pod

=head2 set_retries

Change the retries counter.

=cut
sub set_retries {
	my ( $self , $value ) = @_;
	$self->{'retries'} = $value;
}

=cut

=pod

=head2 get_retries

Get the retries counter.

=cut
sub get_retries {
	my $self = shift;
	return $self->{'retries'};
}

=cut

=pod

=head2 set_expire

Change the expiration date.

=cut
sub set_expire {
	my ( $self , $value ) = @_;
	$self->{'expire'} = $value;
}

=cut

=pod

=head2 get_expire

Get the expiration date.

=cut
sub get_expire {
	my $self = shift;
	return $self->{'expire'};
}

=cut

=pod

=head2 set_persistent

Set the persistance state.

=cut
sub set_persistent {
	my ( $self , $value ) = @_;
	$self->{'persistent'} = $value;
}

=cut

=pod

=head2 get_persistent

Get the persistance state.

=cut
sub get_persistent {
	my $self = shift;
	return $self->{'persistent'};
}

=cut

=pod

=head2 set_valid

Set the certificate validity state.

=cut
sub set_valid {
	my ( $self , $value ) = @_;
	$self->{'valid'} = $value;
}

=cut

=pod

=head2 get_valid

Get the certificate validity state.

=cut
sub get_valid {
	my $self = shift;
	return $self->{'valid'};
}

=cut

=pod

=head2 set_output

Set the certificate error output.

=cut
sub set_output {
	my ( $self , $value ) = @_;
	$self->{'output'} = $value;
}

=cut

=pod

=head2 get_output

Get the certificate error output.

=cut
sub get_output {
	my $self = shift;
	return $self->{'output'};
}

1;
